# ----------------------------------------------------------------------------
# Licensed Materials - Property of IBM
#  
# IBM Websphere Adapter for Ariba Buyer Version 2.0 - GPD5799
#
# (C) Copyright IBM Corp. 2001 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# %Z% %W% %I% %E% %U%
# ----------------------------------------------------------------------------
# 
#   This script is used to create MQSeries repository components for the 
#   variants and partitions of an Ariba Buyer system.
#   Variants and partitions get populated from files found in
#   the /channels/MQSeries/configureOptions subdirectories
#

use English;
use strict;

use Ariba::Config ();
use Ariba::TableEdit ();
use Ariba::Util ();
use File::Basename;


# global variables
my $mappingFileName = "MQCSVMapping.xml";   #MLT
my $mappingFileNameOld = "MQCSVMapping.old";
my $MQSeriesDir    = "channels/MQSeries/";
my $EventTable     = "IntegrationEvent.table";
my $SchemaTable    = "IntegrationSchema.table";
my $ReportMetaFile = "ReportMeta.table";
my $TempExt        = ".tmp";
my $PartitionDir   = "config/partition/"; 
my $VariantDir     = "config/variant/"; 
my $EventTopic     = "MQAribaEvent";
my $EventTopicInit = "MQAribaInitializationEvent";
my $MQLoadFile     = "LoadDBMQSeries.txt";
my $CSVReaderInfo  = "CSVReaderInfo.txt";
my $CSVReaderInfoEnd  = "CSVReaderInfoEnd.txt";
my $EventReaderVar = "EventReaderVar.txt";
my $EventReaderPart= "EventReaderPar.txt";
my $CSVEventMapInfo= "CSVEventMapInfo.txt";
my $EventMapVar    = "CSVEventMapVar.txt";
my $EventMapPart   = "CSVEventMapPart.txt";
my $MapInfoEnd     = "CSVEventMapInfoEnd.txt";
my $PlainNoneEventMap = "PlainNoneEventMap.txt";
my $PlainSuppDirEventMap     = "PlainSuppDirEventMap.txt";
my $ReportMetaMapVar = "ReportMetaEventMap.txt";
my $ReportMetaReaderVar = "ReportMetaEventReader.txt";
my $LogCategories  = "integrationVendor_IBM/info";
# the above line should be a colon separated logging category list

my $SAPConn        = "SAPConn";



#
# The main routine
#
sub main ()
{
    #
    # Parse arguments
    #
    my $action    = $ARGV[0];
    my $basetype  = $ARGV[1];
    my $type      = $ARGV[2];
    my $variant   = $ARGV[3];
    my $partition = $ARGV[4];
    my $itconn    = $ARGV[5];
    my $sapconn   = $ARGV[8];
    if (!defined($action)) {
        usage();
        exit(1);
    }
    if (($action ne "-setup") &&
        ($action ne "-finish") &&
        ($action ne "-mapping") &&
        ($action ne "-verifyMap") &&        
        ($action ne "-migrate") &&
        ($action ne "-enableFeatures_channels") &&
        !defined($type) &&
        !defined($variant)) { 
        usage();
        exit(1);
    }

    if ($action eq "-setup") {
        mergeParameters_tables();
        prependMetaEvents();
    }
    elsif ($action eq "-finish") {
        finish();
    }
    elsif ($action eq "-variant") {
        add_variant($basetype, $type, $variant);
    }
    elsif ($action eq "-partition") {
        add_partition($basetype, $type, $variant, $partition, $itconn, $sapconn);
    }
    elsif ($action eq "-mapping") {
       my $mappingPath = getMappingPath();
       generateMapping($mappingPath);
    }
    elsif ($action eq "-verifyMap") {
       my $mappingPath = $ARGV[1]."/temp";  
       generateMapping($mappingPath);
    }
    elsif ($action eq "-migrate") {
        migrate();
    }
    elsif ($action eq "-enableFeatures_channels") {
        enableFeatures($basetype);
    }
    else {
        usage();
        exit(1);
    } 
}

sub enableFeatures($) {
   my ($feature) = @_;

   #merge feature settings from channels
   Ariba::Config::mergeSettings_ChannelsDefault($feature, $MQSeriesDir, $EventTopicInit);
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# PRQP to GA migration.  Must uninstall PRPQ and install GA code before running
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
sub migrate() {

    mergeParameters_tables();

    mergeIntegrationEvents_table();

    copyMapping();
}




sub mergeIntegrationEvents_table() {

    #Get  partitions & variants from addvariantconfig.txt
    my $configFile = Ariba::Config::FileConfigAddVariantConfig();
    my %genPartitions;
    if (-e $configFile) {
       open(EXISTINGCONFIG, $configFile);
       my $line;
       while ($line = <EXISTINGCONFIG>) {
          chop($line);
          my ($configBaseType, $configType, $configVar, $configPartition, $configItconn, $myTmp1, $myTmp2, $configSAPConn) = split(/ /,$line);
          if ($configVar ne "")  {
             partition_install_events($configType, $configVar, $configPartition, $configItconn, $configSAPConn);
          }

          if ($configBaseType ne "plain" && $configBaseType ne "supplierdirect" ) {
             add_variant_to_report_events ($configVar);
          }
       }
       close(EXISTINGCONFIG);
    }
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Initial setup
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub mergeParameters_tables()
{
    my $src = $MQSeriesDir . 
              Ariba::Config::DirConfigureOptionsInitialConfig() .
              Ariba::Base::FileParameters();
    my $dst = Ariba::Base::FileConfigParameters();
    my $tmp = "";

    # merge Parameters.table settings
    if (-e $src) {
        Ariba::TableEdit::mergeTableFiles($src, $dst, $dst);
        appendStringValue($dst, "", "System.Logging.Console.Categories",
                          $LogCategories);
        appendStringValue($dst, "", "System.Logging.MainLogFile.Categories",
                          $LogCategories);
    }
}
sub  prependMetaEvents()
{
    # prepend MQSeries meta event list into config/LoadDB.txt
    my $src = $MQSeriesDir .
           Ariba::Config::DirConfigureOptionsInitialConfig() .
           $MQLoadFile;
    my $tmp = Ariba::Base::DirConfig() . $MQLoadFile;
    my $dst = Ariba::Config::FileConfigLoadDB(); 
    Ariba::Util::safeCopy($src, $tmp);
    Ariba::Util::appendFile($dst, $tmp);
    Ariba::Util::safeCopy($tmp, $dst);
    Ariba::Util::safeRemove($tmp);

}


#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Finish setup
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub finish()
{
    my $mappingPath = getMappingPath();

    #Generate Mapping file
    generateMapping($mappingPath);
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Partition
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub add_partition ($$$$$)
{
    #bh Defect 242 Start
    #my ($basetype, $type, $variant, $partition, $itconn) = @_;
    my ($basetype, $type, $variant, $partition, $itconn, $sapconn) = @_;
    #bh Defect 242 End


#    check_type($basetype, $type) || 
#        die ("Error: MQSeries channel does not support the type: $type\n");   
    if (!check_type($basetype, $type)) { 
        print ("Warning: MQSeries channel does not support the type: $type\n");
        return;
    }
    #bh Defect 242 Start
    #partition_install_events($type, $variant, $partition, $itconn);
    partition_install_events($type, $variant, $partition, $itconn, $sapconn);
    #bh Defect 242 End
    partition_install_params($basetype, $type, $variant, $partition);
    partition_add_loaddb($basetype, $type, $variant, $partition);
    partition_add_loaddb_language($basetype, $type, $variant, $partition);
}

#
# Install the parameters for IntegrationEvent.table
#
sub partition_install_events ($$$$)
{
    #bh Defect 242 Start
    #my ($type, $variant, $partition, $itconn) = @_;
    my ($type, $variant, $partition, $itconn, $sapconn) = @_;
    #bh Defect 242 End

    my $dir = $MQSeriesDir .
              Ariba::Config::DirConfigureOptions() . $type . "/" .
              $PartitionDir;
    my $src = $dir . $EventTable;
    my $dst =  Ariba::Config::configdir_partition($variant, $partition) .
               $EventTable;
    my $tmp = $src . $TempExt;

    if (-e $src) {
        Ariba::Util::safeCopy($src, $tmp);
        Ariba::Config::replace_partition_string($variant, $partition, $tmp);
        Ariba::Config::replace_interfaceConnection_string($itconn, $tmp);
        Ariba::Config::replace_topic_string($EventTopicInit, $tmp);
        #bh Defect 242 Start
        if ( $sapconn ) {
           Ariba::Config::replace_string($SAPConn, $sapconn, $tmp);
        }
        #bh Defect 242 End
        Ariba::TableEdit::mergeTableFiles($tmp, $dst, $dst);
        Ariba::Config::cleanup_backups($dir);
        Ariba::Util::safeRemove($tmp);
    }
}

#
# Install the MQSeries parameters for this partition in Parameters.table
#
sub partition_install_params ($$$$)
{
    my ($basetype, $type, $variant, $partition) = @_;

    my $dir = $MQSeriesDir . 
              Ariba::Config::DirConfigureOptions() . $type . "/";
    my $src = $dir . Ariba::Base::FileConfigParameters();
    my $dst = Ariba::Base::FileConfigParameters();
    my $tmp = $src . $TempExt;

    if (! -e $src) {
        # perhaps we should be using the basetype file
        $dir = $MQSeriesDir . 
               Ariba::Config::DirConfigureOptions() . $basetype . "/"; 
        $src = $dir . Ariba::Base::FileConfigParameters();
    }

    if (-e $src) {
        Ariba::Util::safeCopy($src, $tmp);
        Ariba::Config::replace_partition_string($variant, $partition, $tmp);
        Ariba::TableEdit::mergeTableFiles($tmp, $dst, $dst);
        Ariba::Config::cleanup_backups($dir);
        Ariba::Util::safeRemove($tmp);
    }
}

#
# Updates LoadDBPartial.txt for this partition
#
sub partition_add_loaddb ($$$$)
{
    my ($basetype, $type, $variant, $partition) = @_;

    my $dir = $MQSeriesDir .
              Ariba::Config::DirConfigureOptions() . $type . "/";
    my $src = $dir . Ariba::Config::FileConfigLoadDB();
    my $dst = Ariba::Config::FileConfigLoadDBPartial();
    my $tmp = $src . $TempExt;


    if (! -e $src) {
        # perhaps we should be using the basetype file
        $dir =  $MQSeriesDir . 
                Ariba::Config::DirConfigureOptions() . $basetype . "/";
        $src = $dir . Ariba::Config::FileConfigLoadDB();
    }

    if (-e $src) {
        Ariba::Util::safeCopy($src, $tmp);
        Ariba::Config::replace_partition_string($variant, $partition, $tmp);
        Ariba::Util::appendFile($tmp, $dst);
        Ariba::Util::safeRemove($tmp);
        Ariba::Config::cleanup_backups($dir);
    }
}

#
# Updates LoadDBLanguagePartial.txt for this partition
#
sub partition_add_loaddb_language ($$$$)
{
    my ($basetype, $type, $variant, $partition) = @_;

    my $dir = $MQSeriesDir .
              Ariba::Config::DirConfigureOptions() . $type . "/";
    my $src = $dir . Ariba::Config::FileConfigLoadDBLanguage();
    my $dst = Ariba::Config::FileConfigLoadDBLanguagePartial();
    my $tmp = $src . $TempExt;

    if (! -e $src) {
        # perhaps we should be using the basetype 
        $dir = $MQSeriesDir . 
               Ariba::Config::DirConfigureOptions() . $basetype . "/";
        $src = $dir . Ariba::Config::FileConfigLoadDBLanguage();
    }

    if (-e $src) {
        Ariba::Util::safeCopy($src, $tmp);
        Ariba::Config::replace_partition_string($variant, $partition, $tmp);
        Ariba::Util::appendFile($tmp, $dst);
        Ariba::Util::safeRemove($tmp);
        Ariba::Config::cleanup_backups($dir);
    }
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Variant
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub add_variant($$$)
{
    my ($basetype, $type, $variant) = @_;

    if (!check_type($basetype, $type)) { 
        return;
    }   

    # Variant Plain isn't quite a first class citizen.
    # Plain should be created implicitly while creating supplierdirect partition
    # Report events are only applicable for variants other than Plain
    if ($variant eq Ariba::Config::VariantPlain()) {
        $type = "plain";
    }
    else
    {
        add_variant_to_report_loaddb($variant);
        add_variant_to_report_events($variant);
    }
    variant_install_schema($type, $variant);
}

#
# Install IntegrationSchema.table changes for new variant
#
sub variant_install_schema ($$)
{
    my ($type, $variant) = @_;

    my $dir = $MQSeriesDir .
              Ariba::Config::DirConfigureOptions() . $type . "/" .
              $VariantDir;
    my $src = $dir . $SchemaTable;
    my $dst = Ariba::Config::configdir_variant($variant) .
              $SchemaTable;
    my $tmp = $src . $TempExt;

    if (-e $src) {
        Ariba::Util::safeCopy($src, $tmp);
        Ariba::Config::replace_variant_string($variant, $tmp);
        Ariba::TableEdit::mergeTableFiles($tmp, $dst, $dst);
        Ariba::Config::cleanup_backups($dir);
        Ariba::Util::safeRemove($tmp);
    }
}

#
# Updates LoadDB.txt for the reports for this variant
#
sub add_variant_to_report_loaddb ($)
{
    my ($variant) = @_;

    # defect 371 Start
    # prepend MQSeries meta event list into config/LoadDBPartial.txt
    my $src = $MQSeriesDir .
           Ariba::Config::DirConfigureOptionsInitialConfig() .
           $MQLoadFile;
    my $tmp = Ariba::Base::DirConfig() . $MQLoadFile;
    my $dst = Ariba::Config::FileConfigLoadDBPartial();
    Ariba::Util::safeCopy($src, $tmp);
    Ariba::Util::appendFile($dst, $tmp);
    Ariba::Util::safeCopy($tmp, $dst);
    Ariba::Util::safeRemove($tmp);
    # defect 371 End

    my $loadDBPartial = Ariba::Config::FileConfigLoadDBPartial();
    open(LOADDB, ">>$loadDBPartial")||die "Couldn't open $loadDBPartial";


    print LOADDB ("None.IntegrationEvent." . $variant . "ReportQuery_csv\n");
    print LOADDB ("None.IntegrationEvent." . $variant . "ReportMeta_csv\n");
    close(LOADDB);
}

#
# Updates None/IntegrationEvent.table for the reports for this variant
#
sub add_variant_to_report_events ($)
{
    my ($variant) = @_;

    my $dir = $MQSeriesDir;
    my $src = $dir . $ReportMetaFile; 
    my $dst = Ariba::Config::configdir_partition(Ariba::Config::VariantPlain(),
              Ariba::Config::PartitionNone()) .
              $EventTable;
    my $tmp = $src . $TempExt;

    Ariba::Util::safeCopy($src, $tmp);
    Ariba::Config::replace_variant_string($variant, $tmp);
    Ariba::Config::replace_topic_string($EventTopic, $tmp);
    Ariba::TableEdit::mergeTableFiles($tmp, $dst, $dst);
    Ariba::Util::safeRemove($tmp);
    Ariba::Config::cleanup_backups($dir);
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Helper to concat string values in table file
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub appendStringValue ($$$$)
{
    my ($filename, $decode, $name, $value) = @_;
    my $oldvalue = Ariba::TableEdit::getValue($filename, $decode, $name);
    if (index($oldvalue, $value) == -1) {
        my $newvalue =  $oldvalue . ":" . $value;
        Ariba::TableEdit::setValue($filename, $decode, $name, $newvalue);
        Ariba::TableEdit::tableEditSaveFile($filename, "current");
    }
}

sub check_type($$)
{
    # check that this channel actually has a configuration option for
    # the specified basetype/type
    my ($basetype, $type) = @_;

    my $basetypePath = $MQSeriesDir .
                       Ariba::Config::DirConfigureOptions() . $basetype;
    my $typePath = $MQSeriesDir .
                   Ariba::Config::DirConfigureOptions() . $type;
    return (-e $basetypePath || -e $typePath);
}

sub usage()
{
        print("MQSeries configuration script\n");
        print("Usage:\n");
        print("config -setup\n");
        print("\tinitial setup for MQSeries configuration\n");
        print("config -finish\n");
        print("\tcomplete setup for MQSeries configuration\n");
        print("config -variant <basetype> <type> <variant>\n");
        print("\tadd variant to MQSeries configuration\n");
        print("config -partition <basetype> <type> <variant> <partition> <connection>\n");
        print("\tadd partition to MQSeries configuration\n");
        print("config -mapping \n");
        print("\tgenerate MQCSVMapping.xml file\n");
        print("\n");
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Generate Mapping file
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# MLT - This subroutine was pretty much completely rewritten to handle putting data
# int seperate directories based on varant and partition.  The old method created one
# huge file with everything and stored it in one place.  This new method only writes 
# the data needed for a variant/partition and stores it it's unique directory:
# <mapping path>/variants/<variant>/partitions/<partition>/MQCSVMapping.xml

sub generateMapping($)
{
    my ($mappingDir) = @_;
 
   # Get variants and partitions
    my $basetypeVar;
    my %basetypeVars = getVariantsPartitions();

    # Find out where MQCSVMapping.xml should be by reading the entry in aqmconfig.xml
    #my $mappingFile = getMappingPath(); 
    #my $mappingDir = getMappingPath();
 

    # getMappingPath() returns the path to a file so we need to add the filename
    my $mappingFile =  $mappingDir."/".$mappingFileName; 

    # Backup existing MQCSVMapping.xml; 
    #my $bck = $mappingFile . ".bak"; 
    #Ariba::Util::safeCopy($mappingFile, $bck); 



    my $var;
    my $VAR;
    my $PART;
    my $part;
    my $basetype;
    my @partArray;
    # For every Variant and partition copy the mapping file to the appropriate directory
    # first create the variant directory
    for $basetypeVar (keys %basetypeVars) {     
	($basetype, $var, @partArray) = (@{$basetypeVars{$basetypeVar}});
	#print "generateMapping: basetype = $basetype, var = $var\n";
	#check to see if variant directory exists, if not create it and subdirectory
	my $tmpDir = $mappingDir."/variants";     
        Ariba::Util::safeMkDir $tmpDir;     

	$tmpDir = $tmpDir."/$var";     
        Ariba::Util::safeMkDir $tmpDir;     

	$tmpDir = $tmpDir."/partitions";     
        Ariba::Util::safeMkDir $tmpDir;     
            
	
        #Now for each partition of this variant, replace and append to mapping file.
        for $part (@partArray) {     
	    ($VAR = $var) =~ tr/a-z/A-Z/;
	    ($PART = $part) =~ tr/a-z/A-Z/;
            my $tmpPartDir = $tmpDir."/$part";     
	    my $dst = $tmpPartDir."/".$mappingFileName;     
	    Ariba::Util::safeMkDir $tmpPartDir;
	    if ( ! -e $dst) {
		# Copy contents of CSVReaderInfo.txt into new MQCSVMapping.xml
		my $src = $MQSeriesDir . $CSVReaderInfo;
		Ariba::Util::safeCopy($src, $dst);
                # Build the CSVReaderInfo data
		createReaderInfo($var, $part, $dst, $basetype);     
		if (($VAR eq "PLAIN") && ($PART eq "NONE")) {
	            	createReportMetaReaderInfo($var, $part, $dst, $basetype, %basetypeVars);
		}
		
		#close out the <CSVReaderInfo> section
		$src = $MQSeriesDir . $CSVReaderInfoEnd;
		Ariba::Util::appendFile($src, $dst);

		# Append contents of CSVEventMapInfo.txt into MQCSVMapping.xml
		$src = $MQSeriesDir . $CSVEventMapInfo;
		Ariba::Util::appendFile($src, $dst);

                # Build the CSVEventMapInfo data
		createEventMaps($var, $part, $dst, $basetype);

		if (($VAR eq "PLAIN") && ($PART eq "NONE")) {
	            	createReportMetaMapInfo($var, $part, $dst, $basetype, %basetypeVars);
		}
		

                # Add the ending elements to MQCSVMapping.xml
		$src = $MQSeriesDir . $MapInfoEnd;
		Ariba::Util::appendFile($src, $dst);
	    }
        }
    }


}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# getVariantsPartitions() - Get a hash of variants and partitions
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# MLT - This sub is a rewrite of the getCSVVariantsPartitions and 
# getSAPVariantsPartitions.   This new version is more generic and returns a
# hash of all the variants and their partitions (variant1=>part1, part2, ...)
sub getVariantsPartitions()
{
    #Get  partitions basetypes & variants from addvariantconfig.txt
    my $configFile = Ariba::Config::FileConfigAddVariantConfig();
    #print "config file: $configFile\n";
    my %genPartitions;
    my %genBasetype;
    if (-e $configFile) {
       open(EXISTINGCONFIG, $configFile);
       my $line;
       while ($line = <EXISTINGCONFIG>) {
          chop($line);
          my ($configBaseType, $configType, $configVar, $configPartition) = split(/ /,$line);
	  #print " partition: bt = $configBaseType, ct = $configType, var = $configVar, part = $configPartition\n";
          if ($configVar ne "")  {
             $genPartitions{$configPartition} = [$configVar];
	     push @{$genPartitions{$configPartition}}, $configBaseType;
          }
       }
       close(EXISTINGCONFIG);
    }

    #Convert Partition=>[Variant,Basetype] Hash to BaseType=>[Variant,Partitions] Hash
    my $part;
    my $var;
    my $basetypeVar;
    my @partBaseArray;
    my $basetype;
    my %genBasetypeVars;
    my %genVars;
    for $part (keys %genPartitions) {     
        @partBaseArray = (@{$genPartitions{$part}}) ;
	($var, $basetype) =  (@partBaseArray);
	$basetypeVar = $basetype.$var;
	#print "part->bt: part = $part, var = $var, bt = $basetype\n";
	if (!(exists $genBasetypeVars{$basetypeVar})) {
	    $genBasetypeVars{$basetypeVar} = [$basetype];
            push @{$genBasetypeVars{$basetypeVar}}, $var;
            push @{$genBasetypeVars{$basetypeVar}}, $part;
	}
        else {
            push @{$genBasetypeVars{$basetypeVar}}, $part;
        }
    }
    return %genBasetypeVars;
}


sub getMappingPath()
{
    #First read AQMSETUPFILE environment variable to read aqmsetup file.
    my $aqmSetupFile = $ENV{AQMSETUPFILE};

    #Now find where aqmconfig.xml file is located by reading aqmsetup file.
    my $aqmConfig;
    my $line;
    if (-e $aqmSetupFile) {
	open(AQMSETUP, $aqmSetupFile);
	while ($line = <AQMSETUP>) {
        if ($line =~ /AQMConfig/) {
		chop($line);
		my @fields = split(/=/,$line);
            $aqmConfig = $fields[1];
		last;
        }
	}
	close(AQMSETUP);
	$aqmConfig = $aqmConfig . "aqmconfig.xml";
    }

    #Next read aqmconfig.xml to find out where the MQCSVMapping.xml file should be
    my $mappingFile;
    if (-e $aqmConfig) {
	open(AQMCONFIG, $aqmConfig) || die("can't open file: $!");
	while ($line = <AQMCONFIG>) {
        if ($line =~ "ePICPropertyFile") {
		chop($line);
		$line =~ s/<ePICPropertyFile>//;
		$line =~ s{</ePICPropertyFile>}();
            $mappingFile = $line;
		last;
        }
	}
	close(AQMCONFIG);
    }
    $mappingFile=~ s/^\s*//;
    return $mappingFile;
}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# createReportMetaReaderInfo($$) - For each variant, replace the @variant@ 
# parameters in CSVEventReaderVar.txt. Append contents to MQCSVMapping.xml
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# MLT - This sub handles the special case of the ReportMeta info for plain variant
sub createReportMetaReaderInfo($$)
{
#createReportMetaReaderInfo($var, $part, $dst, $basetype, %basetypes);
    my ($var, $part, $dst, $basetype, %basetypeVars) = @_;
    my $dir = $MQSeriesDir;
    my $RdrSrc   = $dir . $basetype . $ReportMetaReaderVar;
    my $RdrTmp   = $RdrSrc . $TempExt;
    my $VAR;
    my $basetypeVar;

    my @partArray;

    if (-e $RdrSrc) {
        for $basetypeVar (keys %basetypeVars) {     
	    ($basetype, $var, @partArray) = (@{$basetypeVars{$basetypeVar}});
	    ($VAR = $var) =~ tr/a-z/A-Z/;
	    if ($VAR ne "PLAIN") {
                #print "createReaderInfo:\n     VarSrc: $VarSrc\n     PartSrc: $PartSrc\n\n";
                Ariba::Util::safeCopy($RdrSrc, $RdrTmp);
                Ariba::Config::replace_variant_string($var, $RdrTmp);
                Ariba::Util::appendFile($RdrTmp, $dst);
            }
	}
    }

    Ariba::Config::cleanup_backups($dir);
    Ariba::Util::safeRemove($RdrTmp);

}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# createReportMetaMapInfo($$) - For each variant, replace the @variant@ 
# parameters in CSVEventReaderVar.txt. Append contents to MQCSVMapping.xml
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# MLT - This sub handles the special case of the ReportMeta info for plain variant
sub createReportMetaMapInfo($$)
{
#createReportMetaMapInfo($var, $part, $dst, $basetype, %basetypes);
    my ($var, $part, $dst, $basetype, %basetypeVars) = @_;
    my $dir = $MQSeriesDir;
    my $MapSrc   = $dir . $basetype . $ReportMetaMapVar;
    my $MapTmp   = $MapSrc . $TempExt;
    my @partArray;
    my $VAR;
    my $basetypeVar;

    if (-e $MapSrc) {
        for $basetypeVar (keys %basetypeVars) {     
	    ($basetype, $var, @partArray) = (@{$basetypeVars{$basetypeVar}});
	    ($VAR = $var) =~ tr/a-z/A-Z/;
	    if ($VAR ne "PLAIN") {
                #print "createReaderInfo:\n     VarSrc: $VarSrc\n     PartSrc: $PartSrc\n\n";
                Ariba::Util::safeCopy($MapSrc, $MapTmp);
                Ariba::Config::replace_variant_string($var, $MapTmp);
                Ariba::Util::appendFile($MapTmp, $dst);
	    }
	}
    }

    Ariba::Config::cleanup_backups($dir);
    Ariba::Util::safeRemove($MapTmp);

}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# createReaderInfo($$) - For each variant, replace the @variant@ 
# parameters in CSVEventReaderVar.txt. Append contents to MQCSVMapping.xml
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# MLT - This sub is a rewrite of the original createReaderInfo to handle new input 
# parameters.  CreateReaderInfo was passed a hash and a destination file,
# the new parameters are a variant a partition and a destination file.
sub createReaderInfo($$)
{
    my ($var, $part, $dst, $basetype) = @_;
    my $dir = $MQSeriesDir;
    my $VarSrc   = $dir . $basetype . $EventReaderVar;
    my $PartSrc  = $dir . $basetype . $EventReaderPart;
    my $VarTmp   = $VarSrc . $TempExt;
    my $PartTmp  = $PartSrc . $TempExt;


    #print "createReaderInfo:\n     VarSrc: $VarSrc\n     PartSrc: $PartSrc\n\n";
    #if a event reader var file exists for this variant, replace variables and append to mapping file.
    if (-e $VarSrc) {
        Ariba::Util::safeCopy($VarSrc, $VarTmp);
        Ariba::Config::replace_variant_string($var, $VarTmp);
        Ariba::Util::appendFile($VarTmp, $dst);
    }

    #if a event reader part file exists for this variant, replace variables and append to mapping file.
    if (-e $PartSrc) {
        Ariba::Util::safeCopy($PartSrc, $PartTmp);
        Ariba::Config::replace_string("partition", $part, $PartTmp);
        Ariba::Util::appendFile($PartTmp, $dst);
    }
    Ariba::Config::cleanup_backups($dir);
    Ariba::Util::safeRemove($VarTmp);
    Ariba::Util::safeRemove($PartTmp);

}

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# createEventMaps($$) - For each variant, replace the @variant@ 
# parameters in CSVEventMapVar.txt. Append contents to MQCSVMapping.xml
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# MLT - This sub is a rewrite of the original createEventMaps to handle new input 
# parameters.  CreateReaderInfo was passed a hash and a destination file,
# the new parameters are a variant a partition and a destination file.
sub createEventMaps($$)
{
    my ($var, $part, $dst, $basetype) = @_;

    my $dir      = $MQSeriesDir;
    my $VarSrc   = $dir . $basetype . "EventMapVar.txt";
    my $PartSrc  = $dir . $basetype . "EventMapPart.txt";
    my $PartDefSrc  = $dir . $basetype . "EventMap" . $part . ".txt";
    #print " PartDefSrc: $PartDefSrc \n";
    my $VarTmp   = $VarSrc . $TempExt;
    my $PartTmp  = $PartSrc . $TempExt;


    #if a event map file for this partition exists append it
    if (-e $PartDefSrc) {
	    #print "PartDefSrc file exists!";
	Ariba::Util::appendFile($PartDefSrc, $dst);
    }

    #if a event map file exists for this variant, replace variables and append to mapping file.
    if (-e $VarSrc) {
        Ariba::Util::safeCopy($VarSrc, $VarTmp);
        Ariba::Config::replace_variant_string($var, $VarTmp);
        Ariba::Util::appendFile($VarTmp, $dst);
    }

    #if a event map file exists for this partition, replace variables and append to mapping file.
    if (-e $PartSrc) {
        Ariba::Util::safeCopy($PartSrc, $PartTmp);
        Ariba::Config::replace_string("partition", $part, $PartTmp);
        Ariba::Util::appendFile($PartTmp, $dst);
    }
    Ariba::Config::cleanup_backups($dir);
    Ariba::Util::safeRemove($VarTmp);
    Ariba::Util::safeRemove($PartTmp);

}


#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Copy Mapping file
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub copyMapping()
{   
    # Get variants and partitions
    my $basetypeVar;
    my %basetypeVars = getVariantsPartitions();

    # Find out where MQCSVMapping.xml should be by reading the entry in aqmconfig.xml
    #my $mappingFile = getMappingPath(); 
    my $mappingDir = getMappingPath(); 
    # getMappingPath() returns the path to a file so we need to add the filename
    my $mappingFile =  $mappingDir."/".$mappingFileName; 
    my $oldMappingFile = $mappingDir."/".$mappingFileNameOld;

    # compensation step for error in 2.0.0.1 installer 
    if (! -e $mappingFile) {
       if (-e $oldMappingFile) {
         Ariba::Util::safeCopy($oldMappingFile, $mappingFile);
       } 
    }

    # Backup existing MQCSVMapping.xml; 
    #my $bck = $mappingFile . ".bak"; 
    #Ariba::Util::safeCopy($mappingFile, $bck); 

    my $var;
    my $part;
    my $basetype;
    my @partArray;
    # For every Variant and partition copy the mapping file to the appropriate directory
    # first create the variant directory
    for $basetypeVar (keys %basetypeVars) {     
	($basetype, $var, @partArray) = (@{$basetypeVars{$basetypeVar}});
	#print "generateMapping: basetype = $basetype, var = $var\n";
	#check to see if variant directory exists, if not create it and subdirectory
	my $tmpDir = $mappingDir."/variants";     
        Ariba::Util::safeMkDir $tmpDir;     

	$tmpDir = $tmpDir."/$var";     
        Ariba::Util::safeMkDir $tmpDir;     

	$tmpDir = $tmpDir."/partitions";     
        Ariba::Util::safeMkDir $tmpDir;       
            
	
        #Now for each partition of this variant, replace and append to mapping file.
        for $part (@partArray) {     
            my $tmpPartDir = $tmpDir."/$part";
            my $tmpMappingFile = $tmpPartDir."/".$mappingFileName; 
            Ariba::Util::safeMkDir $tmpPartDir;
            if ( ! -e $tmpMappingFile) {
                Ariba::Util::safeCopy($mappingFile, $tmpPartDir);
            }
        }
    }
    #Ariba::Util::safeRemove($mappingFile);

}



#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Main
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

main();

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# EOF
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
